@import "variables.less";

@body-shadow: inset 20px 0 20px -20px #aaa;

@navbar-default-shadow: none;

@nav-tabs-color: @gray-light;
@nav-tabs-hover-color: darken(@gray-light, 20%);
@nav-tabs-active-color: @gray;

@editor-title-color: @text-color;
@editor-title-bg: #fff;
@editor-title-shadow: none;

@node-title-bg: darken(@body-bg, 5%);
