@font-face {
	font-family: 'djedi';
	src:url('../base/fonts/djedi.eot');
	src:url('../base/fonts/djedi.eot?#iefix') format('embedded-opentype'),
  url('../base/fonts/djedi.ttf') format('truetype'),
	url('../base/fonts/djedi.woff') format('woff'),
	url('../base/fonts/djedi.svg#djedi') format('svg');
	font-weight: normal;
	font-style: normal;
}

[class^="dc-"], [class*=" dc-"] {
	font-family: 'djedi';
	speak: none;
	font-style: normal;
	font-weight: normal;
	font-variant: normal;
	text-transform: none;
	line-height: 1;

	/* Better Font Rendering =========== */
	-webkit-font-smoothing: antialiased;
	-moz-osx-font-smoothing: grayscale;
}

.dc-djedi-icon:before,
.dc-djedi-logo:before {
	content: "\e600";
}
.dc-djedi-logo-inverse:before {
	content: "\e601";
}
.dc-djedi-cms:before {
	content: "\e602";
}
