{% load djedi_admin %}
{% if not exclude_json_nodes %}
<script>window.DJEDI_NODES = {{ json_nodes|safe }};</script>
{% endif %}
{% djedi_xss_domain %}
<style>
#djedi-cms {
  position: fixed;
  top: 0;
  right: -400px;
  width: 400px;
  height: 100%;
  border: 0;
  z-index: 30000;
}
#djedi-cms.fullscreen {
  width: 100%;
}
#djedi-cms.closed {
  opacity: 0.5;
}
#djedi-cms.closed:hover {
  opacity: 1;
}
/* Editable area (outline) */
.djedi-node-outline {
  position: absolute;
  z-index: 20000;
  border-radius: 4px;
  cursor: pointer;
}
.djedi-node-outline:hover,
.djedi-node-outline.visible {
  opacity: 0.25;
  background-color: lightseagreen;
}
.djedi-node-outline.visible:hover {
  opacity: 0.5;
}
.djedi-node-outline.selected {
  opacity: 1;
  background-color: transparent;
  box-shadow: 0px 0px 20px 3px rgba(0, 220, 255, 0.5), inset 0px 0px 0px 2px #00dcff;
}
.djedi-overlay {
  background-color: rgba(0, 0, 0, 0.5);
  position: absolute;
  z-index: 10000;
  display: none;
}
</style>
<iframe id="djedi-cms" src="{{ cms_url }}"></iframe>
